%{open Ast %}

%token BOOL INT FLOAT CHAR STRING FUNC ARRAY CRY
%token LBRACK RBRACK LSQ RSQ LPAREN RPAREN COLON SEMI COMMA DOT
%token IF ELIF ELSE WHILE FOR FOREACH IN AND OR RETURN NOT
%token GT LT LEQ GEQ EQ NEQ
%token PLUS MINUS TIMES DIVIDE ASSN MOD PASSN MASSN DASSN TASSN MOASSN PPLUS MMINUS
%token <int> NUM
%token<float> DEC
%token<char> CHARA
%token<bool> BLIT
%token<string> VAR SLIT
%token EOF

%start program
%type<Ast.program> program

%nonassoc NOELSE
%nonassoc ELSE
%nonassoc ELIF
%right ASSN PASSN MASSN DASSN TASSN MOASSN
%left OR
%left AND
%left EQ NEQ
%left LT GT LEQ GEQ
%left PLUS MINUS
%left TIMES DIVIDE MOD
%right PPLUS MINNUS
%right NOT

%%
program:
    decls EOF { $1 }

decls:
   /*nothing*/ { []}
   | decls fdecl { $2 :: $1 }

fdecl:
    FUNC typ VAR LPAREN RPAREN LBRACK stmt_list RBRACK
        {{typ = $2; fname = $3; formals = []; body = List.rev $7;}}
    | FUNC typ VAR LPAREN RPAREN LBRACK RBRACK
        {{typ = $2; fname = $3; formals = []; body = [];}}
    | FUNC typ VAR LPAREN formal_list RPAREN LBRACK stmt_list RBRACK
        {{typ = $2;
        fname = $3;
        formals = List.rev $5;
        body = List.rev $8;}}

formal_list:
    typ VAR  { [($1, $2)]}
    | formal_list COMMA typ VAR {($3, $4) :: $1}

expr_list:
    /*nothing*/ { [] }
    | expr      { [$1] }
    | expr_list COMMA expr { $3::$1 }

typ:
      INT { Int }
    | FLOAT { Float }
    | CHAR { Char }
    | STRING { String }
    | BOOL { Bool }
    | CRY { Cry }


primitive_typ:
    INT {Int}
    | FLOAT {Float}
    | CHAR { Char }
    | STRING { String }
    | BOOL { Bool }

stmt_list:
    /*nothing {[]}*/
    stmt {[$1]}
    | stmt_list stmt {$2::$1}

stmt:
      expr SEMI { Expr($1) }
    | RETURN expr SEMI         {Return($2)}
    | LBRACK stmt_list RBRACK {Block(List.rev $2)}
    | FOR LPAREN expr SEMI expr SEMI expr RPAREN stmt {For($3, $5, $7, $9 )}
    | WHILE LPAREN expr RPAREN stmt {While($3, $5)}
    | IF LPAREN expr RPAREN stmt ELSE stmt {If($3, $5, $7)}
    | IF LPAREN expr RPAREN stmt %prec NOELSE { If($3, $5, Block([]))}
    | IF LPAREN expr RPAREN stmt elif_else_stmt { If($3, $5, $6)}
    | primitive_typ VAR SEMI { Vdecl($1, $2) }
    | primitive_typ VAR ASSN expr SEMI { VdeclAssign($1, $2, $4) }
    | ARRAY LT INT COMMA NUM GT VAR ASSN LSQ expr_list RSQ SEMI {VdeclAssign(IntList($5), $7, IntListLit($10))}
    | ARRAY LT FLOAT COMMA NUM GT VAR ASSN LSQ expr_list RSQ SEMI {VdeclAssign(FloatList($5), $7, FloatListLit($10))}
    | ARRAY LT STRING COMMA NUM GT VAR ASSN LSQ expr_list RSQ SEMI {VdeclAssign(StringList($5), $7, StringListLit($10))}


elif_else_stmt:
    ELIF LPAREN expr RPAREN stmt elif_else_stmt   {If($3, $5, $6)}
    | ELIF LPAREN expr RPAREN stmt ELSE stmt      {If($3, $5, $7)}



expr:
      NUM   { Num($1) }
    | VAR   { Var($1) }
    | CHARA { Chara($1) }
    | BLIT  { Blit($1) }
    | DEC   { Dec($1) }
    | SLIT  { Slit($1) }
    | LPAREN expr RPAREN { $2 }
    | VAR LSQ expr RSQ {ArrayAccess($1, $3)}
    | expr PLUS expr { Binop($1, Add, $3) }
    | expr MINUS expr { Binop($1, Sub, $3) }
    | expr TIMES expr { Binop($1, Mult, $3) }
    | expr DIVIDE expr { Binop($1, Div, $3) }
    | expr MOD expr    { Binop($1, Mod, $3) }
    | expr EQ expr     { Binop($1, Equal, $3)}
    | expr NEQ expr    {Binop($1, Neq, $3) }
    | expr LT expr   { Binop($1, Less, $3) }
    | expr LEQ expr {Binop($1, Leq, $3) }
    | expr GT expr   {Binop($1, Greater, $3) }
    | expr GEQ expr  {Binop($1, Geq, $3) }
    | expr AND expr  {Binop($1, And, $3) }
    | expr OR expr  {Binop($1, Or, $3) }
    | MINUS expr %prec NOT { Unop(Neg, $2)}
    | NOT expr { Unop(Not, $2)}
    | VAR PPLUS {Assign($1, (Binop(Var($1), Add, Num(1))))}
    | VAR MMINUS {Assign($1, (Binop(Var($1), Sub, Num(1))))}
    | VAR ASSN expr { Assign($1, $3)}
    | VAR PASSN expr {Assign($1, (Binop(Var($1), Add, $3)))}
    | VAR MASSN expr { Assign($1, (Binop(Var($1), Sub, $3)))}
    | VAR TASSN expr { Assign($1, (Binop(Var($1), Mult, $3)))}
    | VAR DASSN expr  { Assign($1, (Binop(Var($1), Div, $3)))}
    | VAR MOASSN expr {Assign($1, (Binop(Var($1), Mod, $3)))}
    | VAR LPAREN args_opt RPAREN { Call($1, $3)}



args_opt:
    /*nothing*/ { []}
    | args_list { List.rev $1 }

args_list:
    | expr     { [$1]}
    | args_list COMMA expr {$3::$1}
