{open Parser}

let digit = ['0'-'9']
let letter = ['a'-'z']
let uletter = ['A' - 'Z']
let ascii = ([' '-'!' '#'-'[' ']'-'~'] | digit)

rule token=
  parse [' ' '\t' '\r' '\n']+        { token lexbuf}
       | eof                         { EOF }
       | "char"                      { CHAR }
       | "int"                       { INT }
       | "str"                       { STRING }
       | "bool"                      { BOOL }
       | "float"                     { FLOAT }
       | "func"                      { FUNC }
       | "array"                     { ARRAY }
       | "for"                       { FOR }
       | "while"                     { WHILE }
       | "if"                        { IF }
       | "else"                      { ELSE }
       | "elif"                      { ELIF }
       | "true"                      { BLIT(true) }
       | "false"                     { BLIT(false) }
       | "return"                    { RETURN }
       | "cry"                       { CRY }
       | "foreach"                   { FOREACH }
       | "in"                        { IN }
       | "and"                       { AND }
       | "or"                        { OR }
       | "not"                       { NOT }
       | '+'                         { PLUS }
       | '-'                         { MINUS }
       | '*'                         { TIMES }
       | '/'                         { DIVIDE }
       | '='                         { ASSN }
       | "%"                         { MOD }
       | "+="                        { PASSN }
       | "-="                        { MASSN }
       | "/="                        { DASSN }
       | "*="                        { TASSN }
       | "%="                        { MOASSN }
       | "++"                        { PPLUS}
       | "--"                        { MMINUS}
       | digit+ as num               { NUM(int_of_string num) }
       | digit+ '.' digit+ as frac   { DEC(float_of_string frac) }
       | letter(letter| uletter| '_')* as id    { VAR(id) }
       | ';'                         { SEMI }
       | ':'                         { COLON }
       | ','                         { COMMA }
       | '('                         { LPAREN }
       | ')'                         { RPAREN }
       | '{'                         { LBRACK }
       | '}'                         { RBRACK }
       | '['                         { LSQ }
       | ']'                         { RSQ }
       | '.'                         { DOT }
       | "=="                        { EQ }
       | "!="                        { NEQ }
       | "<"                         { LT }
       | ">"                         { GT }
       | "<="                        { LEQ }
       | ">="                        { GEQ }
       | '"'([^'"']*)'"' as lxm      { SLIT(lxm) }
       | '''(ascii)''' as char       { CHARA(String.get char 1)}
       |  "/*"                       { comment lexbuf }
       | _ as char                 { raise (Failure("illegal character " ^ Char.escaped char))}
and comment =
    parse "*/"                       { token lexbuf }
        | _                          { comment lexbuf }


(* Add string and char into the scanner *)
